﻿using System;
using System.Globalization;
using System.IO;
using System.Threading;
using System.Windows.Forms;

namespace mp3rename
{
    public partial class MainForm : Form
    {
        #region variable decl.

        public static string SelectedDir;
        public static int FilesProccededTotal;
        public static int FilesProccededSuccesfully;
        private FolderBrowserDialog _rootFolderBrowserDialog;

        #endregion

        public MainForm()
        {
            InitializeComponent();
        }

        public void ChangeFormControlsStatus()
        {
            btn_StartUp.Enabled = !btn_StartUp.Enabled;
        }


        private void WorkThreadFunction()
        {
            try
            {
                RecursiveDirRename(SelectedDir);
            }
            catch (Exception crap)
            {
                Helper.AppendTextToRtB(txt_standardLog,
                                       DateTime.UtcNow.ToLongTimeString() + ": [exception]" + crap.Message +
                                       " on directory " + SelectedDir);
            }
        }


        private void RecursiveDirRename(string rootDir)
        {
            int subDirectoriesProcceded = 0;

            try
            {
                string[] subDirectories = Directory.GetDirectories(rootDir);


                Helper.AppendTextToRtB(txt_standardLog, DateTime.UtcNow.ToLongTimeString() + ": " +
                                                        "[info] root dir: " +
                                                        rootDir + "\n");

                try
                {
                    FileHandler(rootDir);
                }
                catch (Exception crap)
                {
                    Helper.AppendTextToRtB(txt_standardLog, DateTime.UtcNow.ToLongTimeString() +
                                                            ": [exception] " +
                                                            crap.Message + " at root dir " + rootDir + "\n");
                }


                subDirectoriesProcceded++;


                foreach (string currentDirectory in subDirectories)
                {
                    Helper.AppendTextToRtB(txt_standardLog, DateTime.UtcNow.ToLongTimeString() + ": " +
                                                            "[info] dir change: " +
                                                            currentDirectory + "\n");

                    try
                    {
                        FileHandler(currentDirectory);
                    }
                    catch (Exception crap)
                    {
                        Helper.AppendTextToRtB(txt_standardLog, DateTime.UtcNow.ToLongTimeString() +
                                                                ": [exception] " +
                                                                crap.Message + " around directory " + currentDirectory +
                                                                "\n");
                    }


                    //call the method in a sub dir to make sure we get sub-sub-...-dirs as well
                    RecursiveDirRename(currentDirectory);

                    subDirectoriesProcceded++;
                }
            }
            catch (Exception crap)
            {
                Helper.AppendTextToRtB(txt_standardLog, DateTime.UtcNow.ToLongTimeString() +
                                                        ": [exception] " +
                                                        crap.Message + " on path " + rootDir + "\n");
            }


            /* Helper.AppendTextToRtB(txt_standardLog, "\n" + DateTime.UtcNow.ToLongTimeString() +
                                                    ": [finished] " + subDirectoriesProcceded +
                                                    " directories inspected, " + FilesProccededTotal + " files found, " +
                                                    FilesProccededSuccesfully + " renamed! \n"); */
        }

        public void FileHandler(string startFileDirectory)
        {
            string[] fileDirectory = Directory.GetFiles(startFileDirectory);
            foreach (string currentFile in fileDirectory)
            {
                FilesProccededTotal++;

                var file = new FileInfo(currentFile);
                string currentFileName = file.Name;

                long size = 0;
                if (file.Exists)
                {
                    size = file.Length;
                }

                //make sure we don't try to handle small image album art, thumb files, etc
                if (size > 1007146)
                {
                    if (!currentFile.Contains(".mp3") && !currentFile.Contains(".txt") &&
                        !currentFile.Contains(".exe") && !currentFile.Contains(".dll") &&
                        !currentFile.Contains(".jpg") && !currentFile.Contains(".gif") &&
                        !currentFile.Contains(".png") && !currentFile.Contains(".rar") &&
                        !currentFile.Contains(".zip") && !currentFile.Contains(".bmp"))
                    {
                        //need to rename the file else our idtag.dll is gonna go crazy while trying to read the data
                        File.Move(currentFile, currentFile + ".mp3");

                        TagLib.File f = TagLib.File.Create(currentFile + ".mp3");
                        string songName = string.Empty;
                        string artist = string.Empty;
                        string number = string.Empty;


                        //is there even any sign of available id tags in the current file?
                        if (!f.Tag.IsEmpty)
                        {
                            songName = f.Tag.Title;
                            artist = f.Tag.Performers[0];
                            number = f.Tag.Track.ToString();
                            if (int.Parse(number) < 10)
                            {
                                number = "0" + number;
                            }
                        }

                        //got some valid data to rename our file proper?
                        if (!songName.Equals(string.Empty) && !artist.Equals(string.Empty))
                        {
                            string newFileName = number + " - " + artist + " - " + songName + ".mp3";

                            //do we have an allready existing file like our current one?
                            if (File.Exists(startFileDirectory + @"\\\" + newFileName))
                            {
                                newFileName = "duplicate_" + number + " - " + artist + " - " + songName + ".mp3";
                                Helper.AppendTextToRtB(txt_standardLog, DateTime.UtcNow.ToLongTimeString() + ": " +
                                                                        string.Format("[duplicate]: {0}, to: {1}",
                                                                                      currentFileName,
                                                                                      newFileName) + "\n");
                            }

                            try
                            {
                                File.Move(currentFile + ".mp3", startFileDirectory + @"\\\" + newFileName);
                                Helper.AppendTextToRtB(txt_standardLog, DateTime.UtcNow.ToLongTimeString() + ": " +
                                                                        string.Format("[success]: {0} to {1}",
                                                                                      currentFileName,
                                                                                      newFileName) + "\n");
                                FilesProccededSuccesfully++;
                            }
                            catch (IOException iocrap)
                            {
                                // could happen if the file is open by another process, file has been removed, name change, goofy stuff like that
                                Helper.AppendTextToRtB(txt_standardLog, DateTime.UtcNow.ToLongTimeString() + ": " +
                                                                        string.Format("[ioexception]: {0}, to: {1}",
                                                                                      currentFileName,
                                                                                      newFileName) + "\n");
                            }
                        }
                        else
                        {
                            // no valid info found.. sad but lets rename the file to its previous file extension

                            File.Move(currentFile + ".mp3", currentFile);
                            Helper.AppendTextToRtB(txt_standardLog,
                                                   DateTime.UtcNow.ToLongTimeString() + ": " + "[failed] " +
                                                   currentFileName +
                                                   " reason: no id tag found\n");
                        }
                    }
                }
            }
        }

        #region Form controls event handler and mainform loading

        private void MainForm_Load(object sender, EventArgs e)
        {
            txt_standardLog.AppendText(DateTime.UtcNow.ToLongTimeString() +
                                       " ----- short sys info [start] -----\n" +
                                       DateTime.UtcNow.ToLongTimeString() + " language: " +
                                       CultureInfo.CurrentCulture.Name + "\n" +
                                       DateTime.UtcNow.ToLongTimeString() + " cpus: " +
                                       Environment.ProcessorCount + "\n" +
                                       DateTime.UtcNow.ToLongTimeString() +
                                       string.Format(" work dir: {0}", Environment.CurrentDirectory) + "\n" +
                                       DateTime.UtcNow.ToLongTimeString() +
                                       string.Format(" machine name: {0}", Environment.MachineName) + "\n" +
                                       DateTime.UtcNow.ToLongTimeString() +
                                       string.Format(" uptime: {0}", Environment.TickCount/1000/60) + " min." + "\n" +
                                       DateTime.UtcNow.ToLongTimeString() +
                                       string.Format(" system date: {0}", DateTime.Now.ToLongDateString()) + "\n" +
                                       DateTime.UtcNow.ToLongTimeString() +
                                       " ----- short sys info [end] -----" + "\n");

            _rootFolderBrowserDialog = new FolderBrowserDialog
                                           {
                                               Description =
                                                   "Select the start directory where a close inspection is needed.",
                                               ShowNewFolderButton = false,
                                               RootFolder = Environment.SpecialFolder.Desktop
                                           };
        }

        private void btn_StartUp_Click(object sender, EventArgs e)
        {
            ChangeFormControlsStatus();

            // Show the FolderBrowserDialog.
            DialogResult result = _rootFolderBrowserDialog.ShowDialog();
            if (result == DialogResult.OK)
            {
                SelectedDir = _rootFolderBrowserDialog.SelectedPath;
                txt_standardLog.AppendText(DateTime.UtcNow.ToLongTimeString() +
                                           ": root dir: " + SelectedDir + "\n");

                //start renaming in a thread
                var thread = new Thread(WorkThreadFunction);
                thread.Start();
            }
            if (result == DialogResult.Abort || result == DialogResult.Cancel)
            {
                txt_standardLog.AppendText(DateTime.UtcNow.ToLongTimeString() +
                                           ": directory selection aborted.\n");
            }
            else
            {
                txt_standardLog.AppendText(DateTime.UtcNow.ToLongTimeString() +
                                           ": no valid start directory choosen.\n");
            }

            ChangeFormControlsStatus();
        }

        private void btn_FAQ_Click(object sender, EventArgs e)
        {
            MessageBox.Show(
                "This program will try to recover *.mp3 file names through \"ID3-Tags\".\nHit the start button, choose your *.mp3 directory, hit ok and it will start up. \nHope it helps. :) \n\nThanks to the creators of the taglib-sharp libary!",
                "mp3 name recovery - FAQ");
        }

        #endregion
    }
}